/* This file is part of GtkMathView-Bonobo, a Bonobo wrapper for GtkMathView.
 * Copyright (C) 2003 Luca Padovani <lpadovan@cs.unibo.it>
 *                    Pouria Masoudi <pmasoudi@cs.unibo.it>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more information, please visit the project's home page
 * http://helm.cs.unibo.it/gtkmathview-bonobo
 * or send an email to <lpadovan@cs.unibo.it>
 */

#include <config.h>

#include <bonobo.h>
#include <bonobo/bonobo-shlib-factory.h>
#include <gtk/gtkmathview_gmetadom.h>

#include "control-factory.h"
#include "control-data.h"
#include "persist-file.h"
#include "persist-stream.h"
#include "handlers.h"
#include "view.h"

enum
  {
    MATH_VIEW_NAME,
    MATH_VIEW_FIXED,
    MATH_VIEW_WIDTH,
    MATH_VIEW_HEIGHT,
    MATH_VIEW_TOP_X,
    MATH_VIEW_TOP_Y,
    MATH_VIEW_FONT_SIZE,
    MATH_VIEW_VERBOSITY,
    MATH_VIEW_ID_NS_URI,
    MATH_VIEW_ID_NAME
  } math_args;

static void
activate_maction(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  GdomeElement* elem;

  g_return_if_fail(control_data != NULL);

  if (gtk_math_view_get_element_at(control_data->math_view, control_data->x, control_data->y, &elem, NULL, NULL)
      && elem != NULL)
    {
      GdomeException exc = 0;
      GdomeElement* action = find_self_or_ancestor(elem, MATHML_NS_URI, "maction");
      if (action != NULL)
	{
	  gtk_math_view_freeze(control_data->math_view);
	  action_toggle(action);
	  gtk_math_view_thaw(control_data->math_view);
	  gdome_el_unref(action, &exc);
	  g_assert(exc == 0);
	}
      gdome_el_unref(elem, &exc);
      g_assert(exc == 0);
    }
}

static void
copy_link_address(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
}

static void
zoom_in(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  g_return_if_fail(control_data != NULL);
  guint old_font_size = gtk_math_view_get_font_size(control_data->math_view);
  gtk_math_view_set_font_size(control_data->math_view, old_font_size + 1);
}

static void
zoom_out(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  g_return_if_fail(control_data != NULL);
  guint old_font_size = gtk_math_view_get_font_size(control_data->math_view);
  gtk_math_view_set_font_size(control_data->math_view, old_font_size - 1);
}

static void
switch_semantic_selection(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  g_return_if_fail(control_data != NULL);
  control_data->semantic_selection = !control_data->semantic_selection;
  if (control_data->semantic_selection)
    {
      GdomeElement* elem = find_element_with_id(control_data->root_selected,
						control_data->id_ns_uri,
						control_data->id_name);
    }
}

/*
static void
copy_selected_markup(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  gchar* res = copy_markup(control_data->root_selected);
  if (res != NULL)
    {
      set_clipboard(res);
      g_free(res);
    }
}
*/

static void
copy_selected_id(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
}

static void
select_parent(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  g_return_if_fail(control_data != NULL);
  if (control_data->root_selected != NULL)
    {
      GdomeException exc = 0;
      GdomeNode* parent_node = gdome_el_parentNode(control_data->root_selected, &exc);
      g_assert(parent_node != NULL);
      g_assert(exc == 0);
      GdomeElement* parent = gdome_cast_el(parent_node);
      gtk_math_view_control_data_set_root_selected(control_data, parent);
      gdome_n_unref(parent_node, &exc);
      g_assert(exc == 0);
    }
}

static void
deselect(GtkMathViewControlData* control_data, guint action, GtkWidget* widget)
{
  g_return_if_fail(control_data != NULL);

  if (control_data->root_selected != NULL)
    {
      gtk_math_view_unselect(control_data->math_view, control_data->root_selected);
      control_data->root_selected = NULL;
    }
}

/* Our menu, an array of GtkItemFactoryEntry structures that defines each menu item */
static GtkItemFactoryEntry menu_items[] = {
  { "/Activate <maction>", NULL,    activate_maction, 0, "<StockItem>", GTK_STOCK_EXECUTE },
  { "/Copy Link Address",  NULL,    copy_link_address, 0, "<StockItem>", GTK_STOCK_COPY },
  /*  { "/Jump To",       NULL,         NULL,           0, "<StockItem>", GTK_STOCK_JUMP_TO }, */
  { "/sep1",          NULL,         NULL,           0, "<Separator>" },
  /* { "/Semantic Selection", NULL,    switch_semantic_selection, 0, "<CheckItem>" }, */
  /* { "/Copy Selected Markup", NULL,  copy_selected_markup, 0, "<Item>" }, */
  /* { "/Copy Id",       NULL,         copy_selected_id, 0, "<Item>" }, */
  /*  { "/Show Selected", NULL,         NULL,           0, "<Item>" }, */
  { "/Select Parent", NULL,         select_parent,  0, "<Item>" },
  { "/De-Select",     NULL,         deselect, 0, "<StockItem>", GTK_STOCK_CLEAR },
  { "/sep2",          NULL,         NULL,           0, "<Separator>" },
  { "/Smaller",       NULL,         zoom_out,       0, "<StockItem>", GTK_STOCK_ZOOM_OUT },
  { "/Bigger",        NULL,         zoom_in,        0, "<StockItem>", GTK_STOCK_ZOOM_IN },
  /* { "/Properties...", NULL,         NULL,           0, "<StockItem>", GTK_STOCK_PROPERTIES }, */
};

static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

gboolean
button_pressed_cb(GtkMathView* math_view, GdkEventButton* event, GtkMathViewControlData* control_data)
{
  g_return_val_if_fail(math_view != NULL, FALSE);
  g_return_val_if_fail(event != NULL, FALSE);
  g_return_val_if_fail(control_data != NULL, FALSE);

  if (event->button == 3)
    {
#if 0
      gtk_menu_popup (GTK_MENU(control_data->popup_menu), NULL, NULL,
		      NULL, event, event->button, event->time);
#endif
      control_data->x = (gint) event->x;
      control_data->y = (gint) event->y;
      printf("data %d %d\n", control_data->x, control_data->y);
      
      gtk_item_factory_popup_with_data(control_data->item_factory,
				       NULL, NULL, event->x_root, event->y_root,
				       event->button, gtk_get_current_event_time());
    }

  return FALSE;
}


static void
get_prop(BonoboPropertyBag* bag,
	 BonoboArg* arg,
	 guint arg_id,
	 CORBA_Environment *ev,
	 GtkMathViewControlData* control_data)
{
  g_assert(control_data != NULL);

  switch (arg_id)
    {
    case MATH_VIEW_WIDTH:
      {
	gint width;
	gtk_math_view_get_size(control_data->math_view, &width, NULL);
	BONOBO_ARG_SET_INT(arg, width);
      }
      break;
    case MATH_VIEW_HEIGHT:
      {
	gint height;
	gtk_math_view_get_size(control_data->math_view, NULL, &height);
	BONOBO_ARG_SET_INT(arg, height);
      }
      break;
    case MATH_VIEW_TOP_X:
      {
	gint top_x;
	gtk_math_view_get_top(control_data->math_view, &top_x, NULL);
	BONOBO_ARG_SET_INT(arg, top_x);
      }
      break;
    case MATH_VIEW_TOP_Y:
      {
	gint top_y;
	gtk_math_view_get_top(control_data->math_view, NULL, &top_y);
	BONOBO_ARG_SET_INT(arg, top_y);
      }		
      break;
    case MATH_VIEW_FONT_SIZE:
      BONOBO_ARG_SET_INT(arg, gtk_math_view_get_font_size(control_data->math_view));
      break;
    case MATH_VIEW_VERBOSITY:
      BONOBO_ARG_SET_INT(arg, gtk_math_view_get_log_verbosity(control_data->math_view));
      break;
    case MATH_VIEW_ID_NS_URI:
      {
	gchar* id_ns_uri = gtk_math_view_control_data_get_id_ns_uri(control_data);
	BONOBO_ARG_SET_STRING(arg, id_ns_uri);
	g_free(id_ns_uri);
      }
      break;
    case MATH_VIEW_ID_NAME:
      {
	gchar* id_name = gtk_math_view_control_data_get_id_name(control_data);
	BONOBO_ARG_SET_STRING(arg, id_name);
	g_free(id_name);
      }
      break;
    default:
      bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
      break;
    }
}

static void
set_prop(BonoboPropertyBag* bag,
	 const BonoboArg* arg,
	 guint arg_id,
	 CORBA_Environment* ev,
	 GtkMathViewControlData* control_data)
{
  g_assert(control_data != NULL);

  switch (arg_id)
    {
    case MATH_VIEW_TOP_X:
      {
	guint old_top_y;
	gtk_math_view_get_top(control_data->math_view, NULL, &old_top_y);
	gtk_math_view_set_top(control_data->math_view, BONOBO_ARG_GET_INT(arg), old_top_y);
      }
      break;
    case MATH_VIEW_TOP_Y:
      {
	guint old_top_x;
	gtk_math_view_get_top(control_data->math_view, &old_top_x, NULL);
	gtk_math_view_set_top(control_data->math_view, BONOBO_ARG_GET_INT(arg), old_top_x);
      }
      break;
    case MATH_VIEW_FONT_SIZE:
      fprintf(stderr, "setting font size to %d\n", BONOBO_ARG_GET_INT(arg));
      gtk_math_view_set_font_size(control_data->math_view, BONOBO_ARG_GET_INT(arg));
      break;
    case MATH_VIEW_VERBOSITY:
      gtk_math_view_set_log_verbosity(control_data->math_view, BONOBO_ARG_GET_INT(arg));
      break;
    case MATH_VIEW_ID_NS_URI:
      gtk_math_view_control_data_set_id_ns_uri(control_data, BONOBO_ARG_GET_STRING(arg));
      break;
    case MATH_VIEW_ID_NAME:
      gtk_math_view_control_data_set_id_name(control_data, BONOBO_ARG_GET_STRING(arg));
      break;
    default:
      bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
      break;
    }
}

static void 
control_destroy(BonoboObject *object, GtkMathViewControlData *cd)
{
  gtk_math_view_control_data_destroy(cd);
}

static void 
gtk_math_view_control_init(BonoboControl *control, GtkWidget *math_view)
{
  GtkMathViewControlData *control_data;
  GtkItemFactory *item_factory;
  
  Bonobo_UIContainer remote_ui_container;
  BonoboUIComponent *ui_component;
  
  BonoboPropertyBag *prop_bag;
  BonoboObject *persist_file;
  BonoboObject *persist_stream;
  BonoboEventSource *evs;
  View* view;

  control_data = gtk_math_view_control_data_new(control, (GtkMathView*)math_view);
  
  g_signal_connect (control, "destroy", G_CALLBACK (control_destroy), control_data);
  
  /* Same as before but don't bother with the accelerators */
  control_data->item_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
  gtk_item_factory_create_items (control_data->item_factory, nmenu_items, menu_items, control_data);
  /* control_data->popup_menu = gtk_item_factory_get_widget (control_data->item_factory, "<main>"); */
  /* gtk_widget_ref(control_data->popup_menu); */
  
  evs = bonobo_event_source_new();
  bonobo_object_add_interface(BONOBO_OBJECT(control), BONOBO_OBJECT(evs));

  view = view_new(control_data);
  bonobo_object_add_interface(BONOBO_OBJECT(control), BONOBO_OBJECT(view));

  persist_file = gtk_math_view_persist_file_new(GTK_MATH_VIEW(math_view));
  bonobo_object_add_interface(BONOBO_OBJECT(control), persist_file);

  persist_stream = gtk_math_view_persist_stream_new(GTK_MATH_VIEW(math_view));
  bonobo_object_add_interface(BONOBO_OBJECT(control), persist_stream);

  prop_bag = bonobo_property_bag_new(get_prop, set_prop, control_data);
  bonobo_control_set_properties(control, BONOBO_OBJREF(prop_bag), NULL);
	
  bonobo_property_bag_add(prop_bag, "width",
			  MATH_VIEW_WIDTH, BONOBO_ARG_INT,
			  NULL,
			  "Width of the view", 0);
  bonobo_property_bag_add(prop_bag, "height",
			  MATH_VIEW_HEIGHT, BONOBO_ARG_INT,
			  NULL,
			  "Height of the view", 0);
  bonobo_property_bag_add(prop_bag, "top-x",
			  MATH_VIEW_TOP_X, BONOBO_ARG_INT,
			  NULL,
			  "X coordinate of the top-left corner", 0);
  bonobo_property_bag_add(prop_bag, "top-y",
			  MATH_VIEW_TOP_Y, BONOBO_ARG_INT,
			  NULL, 
			  "Y coordinate of the top-left corner", 0);
  bonobo_property_bag_add(prop_bag, "font-size",
			  MATH_VIEW_FONT_SIZE, BONOBO_ARG_INT,
			  NULL,
			  "Default font size", 0);
  bonobo_property_bag_add(prop_bag,"verbosity",
			  MATH_VIEW_VERBOSITY, BONOBO_ARG_INT,
			  NULL,
			  "Verbosity level", 0);
  bonobo_property_bag_add(prop_bag, "id-ns-uri",
			  MATH_VIEW_ID_NS_URI, BONOBO_ARG_STRING,
			  NULL,
			  "Namespace URI of ID attribute", 0);
  bonobo_property_bag_add(prop_bag, "id-name",
			  MATH_VIEW_ID_NAME, BONOBO_ARG_STRING,
			  NULL,
			  "Name of ID attribute", 0);

  bonobo_object_unref(BONOBO_OBJECT(prop_bag));
  
  g_signal_connect(control,"set_frame", G_CALLBACK(set_frame), control_data);
  /* we use _after so that this signal handler is called AFTER the default
   * handler */
  g_signal_connect_after(control_data->math_view, "button_press_event", G_CALLBACK (button_pressed_cb), control_data);
  g_signal_connect(control_data->math_view, "click", G_CALLBACK (click_cb), control_data);
  g_signal_connect(control_data->math_view, "select_begin", G_CALLBACK(select_begin_cb), control_data);
  g_signal_connect(control_data->math_view, "select_over", G_CALLBACK(select_over_cb), control_data);
  g_signal_connect(control_data->math_view, "select_end", G_CALLBACK(select_end_cb), control_data);
  g_signal_connect(control_data->math_view, "select_abort", G_CALLBACK(select_abort_cb), control_data);
}

static BonoboObject*
gtk_math_view_control_factory(BonoboGenericFactory* factory, const gchar* component_id,
			      gpointer closure)
{
  BonoboControl *control;
  GtkMathView* math_view;
  GtkWidget *scrolled_window;

  math_view = gtk_math_view_new(NULL,NULL);
  gtk_widget_show(math_view);
  
#if 1
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_add(GTK_CONTAINER (scrolled_window), GTK_WIDGET (math_view));

  /* putting SHADOW_NONE screws the plugin window, how's that??? */
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_NONE);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_window),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show_all (scrolled_window);
  control = bonobo_control_new(scrolled_window);
#else
  control = bonobo_control_new(math_view);
#endif
  if(control)
    {
      gtk_math_view_control_init(control, math_view);
      return BONOBO_OBJECT(control);
    }
  else
    return NULL;
}

BONOBO_ACTIVATION_SHLIB_FACTORY (CONTROL_FACTORY_ID, "GtkMathView Factory",
				 gtk_math_view_control_factory, NULL);

